@@ -51,21 +51,20 @@ def send_http_header(octr):
 class page_framework(object):
     get_server_url = tools.mk_not_imp('get_server_url')
     
-    def __init__(self, skel_page):
+    def __init__(self, skel_page, db):
 	super(page_framework, self).__init__()
 	self._skel_page = skel_page
+        self._db = db
 	pass
     
     def get_last_docs(self):
-	import GinGin_db
-	docs = GinGin_db.get_last_docs(GINGIN_MOST_RECENT_N)
+	docs = self._db.get_last_docs(GINGIN_MOST_RECENT_N)
 	r = [{'id': doc[0], 'title': doc[1]} for doc in docs]
 	return r
     
     def get_hot_keywords(self):
-	import GinGin_db
 	from xml_template import escape_cdata
-	hot_kws = GinGin_db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
+	hot_kws = self._db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
 	r  = [{'name': kw[0], 'num': kw[1]} for kw in hot_kws]
 	return r
     
@@ -103,17 +102,44 @@ class page_framework(object):
 	pass
     pass
 
-def GinGin(req):
+def GinGin(req, db):
 
     # actin_out is the output of request action.
     # If it is None, the action had generated output page for them-self,
     # or action_out comprises data, template, and, maybe, page skelelton
     # waiting for rendering.  Here, we render the data for it.
     form = req.getFieldStorage()
+    octr = output_ctr(req.out, req.env)
     try:
-        octr = output_ctr(req.out, req.env)
         GinGin_acts.str = tools.hack_str
-	action_out = GinGin_acts.GinGin_run_action(form, req.env, octr)
+	action_out = GinGin_acts.GinGin_run_action(form, req.env, octr, db)
+        if action_out != None:  # There is data waiting for rendering
+            # Render the data.
+            from templates import gingin
+            
+            # page_skel is outest skeleton of output page.
+            # data is data being showed by output page.
+            # temp is the template used to render data.
+            #
+            # The result of rendering is ovened with page_skel to create
+            # output page.
+            if len(action_out) == 2:
+                data, temp = action_out
+                page_skel = gingin.gingin
+                gingin.str = tools.hack_str
+                pass
+            elif len(action_out) == 3:  # actin_out is with page_skel
+                data, temp, page_skel = action_out
+                pass
+            pfwrk = page_framework(page_skel, db)
+            pfwrk.get_server_url = tools.get_server_url
+            send_http_header(octr)
+            octr.ofo.write("<?xml version=\"1.0\" encoding=\"%s\"?>\n" % (octr.charset,))
+            if octr.media_type == 'text/html':
+                octr.ofo.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')
+                pass
+            pfwrk(data, temp, octr, req.env)
+            pass
     except:
 	import traceback
 	tb_str = traceback.format_exc()
@@ -123,42 +149,21 @@ def GinGin(req):
         send_http_header(octr)
         temp.gen_doc(data)
         sys.exit(0)
-	
-    if action_out != None:       # There is data waiting for rendering
-        # Render the data.
-        from templates import gingin
-        
-        # page_skel is outest skeleton of output page.
-        # data is data being showed by output page.
-        # temp is the template used to render data.
-        #
-        # The result of rendering is ovened with page_skel to create
-        # output page.
-        if len(action_out) == 2:
-            data, temp = action_out
-            page_skel = gingin.gingin
-            gingin.str = tools.hack_str
-        elif len(action_out) == 3:  # actin_out is with page_skel
-            data, temp, page_skel = action_out
-            pass
-        pfwrk = page_framework(page_skel)
-        pfwrk.get_server_url = tools.get_server_url
-        send_http_header(octr)
-        octr.ofo.write("<?xml version=\"1.0\" encoding=\"%s\"?>\n" % (octr.charset,))
-        if octr.media_type == 'text/html':
-            octr.ofo.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')
-            pass
-        pfwrk(data, temp, octr, req.env)
-        #sys.exit(0)
         pass
+    
     pass
 
 if __name__ == '__main__':
     import fcgi
+    import GinGin_db
+    import config
 
+    ggdb = GinGin_db.GGDB(GINGIN_DB)
+    config.GINGIN_USER.set_db(ggdb)
+    
     while fcgi.isFCGI():
         req = fcgi.Accept()
-        GinGin(req)
+        GinGin(req, ggdb)
         req.Finish()
         pass
     pass
