@@ -108,12 +108,13 @@ class DENY_ACTION:
 #
 # A new_action is a callable that being called to perform actions.
 class new_action(object):
-    def __init__(self, act_name, ival=[], fval=[]):
+    def __init__(self, act_name, ival=[], fval=[], db=None):
 	super(new_action, self).__init__()
 	self._act_name = act_name
 	self._ival = ival
         self._fval = fval
 	self.encoding = GINGIN_ENCODING
+        self.db = db
 	pass
 
     ## Authorize the user to access a document.
@@ -192,21 +193,50 @@ class CGI_form(action_form):
 		pass
 	    pass
 	pass
+
+    def _get_browser(self):
+        try:
+            agent = self.env['HTTP_USER_AGENT']
+            if agent.find('Firefox/') >= 0:
+                browser_vers = [browser_ver.split('/')
+                                for p in agent.strip().split(' ')
+                                if '/' in p]
+                for browser, ver in browser_vers:
+                    if browser == 'Firefox':
+                        ver = tuple([int(x) for x in ver.split('.')])
+                        return browser, ver
+                    pass
+            else:
+                i = agent.find('MSIE ')
+                i = i + 5
+                for j in range(i, len(agent)):
+                    if agent[j] not in '0123456779.':
+                        ver = tuple([int(x) for x in agent[i:j].split('.')])
+                        return ('MSIE', ver)
+                    pass
+                pass
+        except:
+            pass
+        
+        return ('Unknown', (0,))
     
     def __full_with_path_info(self, res):
-	encoding = GINGIN_ENCODING
-	try:
-	    if self.env['HTTP_USER_AGENT'].find(' MSIE ') >= 0:
-		encoding = 'utf8'
-	except:
-	    pass
+        browser, ver = self._get_browser()
+        if browser != 'Firefox' or ver >= (3, 0):
+            # All URLs are transform to UTF-8 for Firefox/3.0+ or MSIE
+            encoding = 'utf8'
+        else:
+            encoding = GINGIN_ENCODING
+            pass
+        
 	try:
 	    pinfo = split(self.env['PATH_INFO'], '/')[2:]
 	    i = 0
 	    while i < len(pinfo) and i < len(self._ival):
 		var = self._ival[i]
-		res[var] = self.to_utf8(pinfo[i], encoding)
-		i = i + 1
+                res[var] = self.to_utf8(pinfo[i], encoding)
+                i = i + 1
+                pass
 	except KeyError:
 	    pass
 	pass
@@ -309,8 +339,8 @@ class test_octr(action_octr):
 # @{
 
 class default_act(new_action):
-    def __init__(self):
-        super(default_act, self).__init__('default_act',  ['func'])
+    def __init__(self, db):
+        super(default_act, self).__init__('default_act',  ['func'], db=db)
         pass
     
     def __call__(self):
@@ -330,8 +360,8 @@ class default_act(new_action):
 
 ## Show the document with specified ID.
 class show_id_doc(new_action):
-    def __init__(self):
-	super(show_id_doc, self).__init__('show_id_doc',  ['doc_id'])
+    def __init__(self, db):
+	super(show_id_doc, self).__init__('show_id_doc',  ['doc_id'], db=db)
 	pass
 	
     def __call__(self):
@@ -343,13 +373,11 @@ class show_id_doc(new_action):
         self.auth(doc_id)
 	
 	kws, doc_id, title, doc_body, pdate, uid, published = \
-            GinGin_db.get_id_doc(doc_id)
+            self.db.get_id_doc_formated(doc_id)
 	content = {}
 	creator = GINGIN_USER.get_user_from_id(self, uid)
-	all_kws = GinGin_db.get_all_kws()
-	doc_body = GinGin_db.replace_kws(doc_body, all_kws)
 	
-	comments = GinGin_db.get_id_doc_comments(doc_id)
+	comments = self.db.get_id_doc_comments(doc_id)
 	comments = [{'msg': x[4],
                      'website': escape_url(x[3]),
                      'pdate': x[5],
@@ -373,8 +401,10 @@ class show_id_doc(new_action):
 
 ## Show the source of the document of specified ID.
 class show_id_doc_src(new_action):
-    def __init__(self):
-	super(show_id_doc_src, self).__init__('show_id_doc_src',  ['doc_id'])
+    def __init__(self, db):
+	super(show_id_doc_src, self).__init__('show_id_doc_src',
+                                              ['doc_id'], db=db)
+        pass
 	
     def __call__(self):
 	doc_id = int(self.form['doc_id'])
@@ -382,7 +412,7 @@ class show_id_doc_src(new_action):
         self.auth(doc_id)
 
 	kws, doc_id, title, doc_body, pdate, uid, published = \
-            GinGin_db.get_id_doc(doc_id)
+            self.db.get_id_doc(doc_id)
 	creator = GINGIN_USER.get_user_from_id(self, uid)
 	represent.show_doc_src(kws, doc_id, title, doc_body, pdate, creator)
 	pass
@@ -391,8 +421,8 @@ class show_id_doc_src(new_action):
 	
 ## Show editor form for the document with specified ID.
 class edit_id_doc(new_action):
-    def __init__(self):
-	super(edit_id_doc, self).__init__('edit_id_doc', ['doc_id'])
+    def __init__(self, db):
+	super(edit_id_doc, self).__init__('edit_id_doc', ['doc_id'], db=db)
 	pass
 	
     def __call__(self):
@@ -401,7 +431,7 @@ class edit_id_doc(new_action):
         self.auth(doc_id)
 	
 	kws, doc_id, title, doc_body, pdate, uid, published = \
-            GinGin_db.get_id_doc(doc_id)
+            self.db.get_id_doc(doc_id)
 	
 	content = {}
 	content['keywords'] = [{'keyword': kw} for kw in kws]
@@ -418,8 +448,9 @@ class edit_id_doc(new_action):
 
 ## Show unpublished docs.
 class show_all_unpublished(new_action):
-    def __init__(self):
-        super(show_all_unpublished, self).__init__('show_all_unpublished', [])
+    def __init__(self, db):
+        super(show_all_unpublished, self).__init__('show_all_unpublished',
+                                                   [], db=db)
         pass
 
     def __call__(self):
@@ -427,7 +458,7 @@ class show_all_unpublished(new_action):
 
         user = self.auth()
 
-        docs = GinGin_db.get_unpublished_docs()
+        docs = self.db.get_unpublished_docs()
         docs = [{'doc_id': doc[0], 'title': doc[1],
                  'pdate': doc[2], 'creator': doc[3]}
                 for doc in docs]
@@ -438,14 +469,14 @@ class show_all_unpublished(new_action):
 
 ## Show all keywords of a GinGin system.
 class show_all_kws(new_action):
-    def __init__(self):
-	super(show_all_kws, self).__init__('show_all_kws', [])
+    def __init__(self, db):
+	super(show_all_kws, self).__init__('show_all_kws', [], db=db)
 	pass
     
     def __call__(self):
 	from templates import all_kws
 	
-	kws = GinGin_db.get_all_kws()
+	kws = self.db.get_all_kws()
 	kws = [{'keyword': kw} for kw in kws]
 	content = {'title': 'All Keywords', 'keywords': kws, 'keywords_num': len(kws)}
 	
@@ -457,11 +488,12 @@ class show_all_kws(new_action):
 
 ## Update content of the document specified ID.
 class update(new_action):
-    def __init__(self):
+    def __init__(self, db):
 	super(update, self).__init__('update',
-                                         ['doc_id', 'title', 'keywords',
-                                          'doc_body', 'update_pdate',
-                                          'published'])
+                                     ['doc_id', 'title', 'keywords',
+                                      'doc_body', 'update_pdate',
+                                      'published'],
+                                     db=db)
         pass
     
     def __call__(self):
@@ -507,7 +539,7 @@ class update(new_action):
 	if len(doc_body) > MAX_DOC_SZ:
 	    return represent.error_page('too long', self.octr.ofo)
         uid = GINGIN_USER.get_id_from_user(self, user)
-        doc_id = GinGin_db.update_doc(doc_id, title, kws, doc_body,
+        doc_id = self.db.update_doc(doc_id, title, kws, doc_body,
                                       uid, update_pdate, published)
         base, spath = get_server_url(self.octr.env)
         represent.show_update_ok(doc_id, title, spath)
@@ -515,8 +547,9 @@ class update(new_action):
 
 ## Show documents and URLs of a keyword.
 class show_kw_docs(new_action):
-    def __init__(self):
-	super(show_kw_docs, self).__init__('show_kw_docs', ['keyword'])
+    def __init__(self, db):
+	super(show_kw_docs, self).__init__('show_kw_docs', ['keyword'],
+                                           db=db)
 	pass
     
     def __call__(self):
@@ -524,8 +557,8 @@ class show_kw_docs(new_action):
 	from xml_template import escape_url
 	
 	kw = self.form['keyword'].decode('utf-8')
-	docs = GinGin_db.get_kw_docs(kw)
-	urls = GinGin_db.get_kw_urls(kw)
+	docs = self.db.get_kw_docs(kw)
+	urls = self.db.get_kw_urls(kw)
 	if len(docs) > 1 or len(urls) > 0:
 	    docs =[{'doc_id': d[0], 'title': d[1], 'pdate': d[2], 'creator': d[3]} for d in docs]
 	    urls = [{'url_id': u[0], 'title': u[1], 'url': escape_url(u[2]), 'abstract': u[3], 'pdate': u[4], 'creator': u[5]} for u in urls]
@@ -545,8 +578,8 @@ class show_kw_docs(new_action):
 
 ## Show a form for a new document.
 class new_doc(new_action):
-    def __init__(self):
-	super(new_doc, self).__init__('new_doc', [])
+    def __init__(self, db):
+	super(new_doc, self).__init__('new_doc', [], db=db)
 	pass
 	
     def __call__(self):
@@ -559,10 +592,11 @@ class new_doc(new_action):
 
 ## Upload an attachment.
 class add_afile(new_action):
-    def __init__(self):
+    def __init__(self, db):
         super(add_afile, self).__init__('add_afile',
                                         ['doc_id', 'afkey'],
-                                        ['afile'])
+                                        ['afile'],
+                                        db=db)
         pass
 	
     def __call__(self):
@@ -573,14 +607,15 @@ class add_afile(new_action):
 	afkey = form['afkey']
 	afobj = form['afile'].file
 	mtype = form['afile'].type
-	GinGin_db.add_afile_to_doc(doc_id, afkey, afobj, mtype)
+	self.db.add_afile_to_doc(doc_id, afkey, afobj, mtype)
 	represent.go_back(self.env)
         pass
 
 ## Delete an attachment.
 class del_afile(new_action):
-    def __init__(self):
-	super(del_afile, self).__init__('del_afile', ['doc_id', 'afkey'])
+    def __init__(self, db):
+	super(del_afile, self).__init__('del_afile', ['doc_id', 'afkey'],
+                                        db=db)
         pass
 	
     def __call__(self):
@@ -589,14 +624,15 @@ class del_afile(new_action):
 	self.auth(doc_id)
 	
 	afkey = form['afkey']
-	GinGin_db.remove_afile_by_afkey(doc_id, afkey)
+	self.db.remove_afile_by_afkey(doc_id, afkey)
 	represent.go_back(self.env)
         pass
 	
 ## Get content of an attachement.
 class get_afile(new_action):
-    def __init__(self):
-	super(get_afile, self).__init__('get_afile', ['doc_id', 'afkey'])
+    def __init__(self, db):
+	super(get_afile, self).__init__('get_afile', ['doc_id', 'afkey'],
+                                        db=db)
 	
     def __call__(self):
         form = self.form
@@ -605,7 +641,7 @@ class get_afile(new_action):
 	
 	afkey = form['afkey']
 	afname, mtype = \
-            GinGin_db.get_afname_mtype_by_afkey(doc_id, afkey)
+            self.db.get_afname_mtype_by_afkey(doc_id, afkey)
 	if afname != None:
 	    represent.send_file(afname, afkey, mtype)
 	else:
@@ -616,8 +652,8 @@ class get_afile(new_action):
 
 ## Show attachments of the document with specified ID.
 class show_afiles(new_action):
-    def __init__(self):
-	super(show_afiles, self).__init__('show_afiles', ['doc_id'])
+    def __init__(self, db):
+	super(show_afiles, self).__init__('show_afiles', ['doc_id'], db=db)
         pass
 	
     def __call__(self):
@@ -627,7 +663,7 @@ class show_afiles(new_action):
 	doc_id = int(form['doc_id'])
         self.auth(doc_id)
 
-	afkeys = GinGin_db.get_afkeys_doc(doc_id)
+	afkeys = self.db.get_afkeys_doc(doc_id)
         content = {'doc_id': doc_id,
                    'title': 'Attachments of doc #%d' % (doc_id)}
         _attaches = [{'afkey': afkey} for afkey in afkeys]
@@ -640,8 +676,9 @@ class show_afiles(new_action):
 
 ## Show URLs with specified keyword.
 class show_kw_urls(new_action):
-    def __init__(self):
-	super(show_kw_urls, self).__init__('show_kw_urls', ['keyword'])
+    def __init__(self, db):
+	super(show_kw_urls, self).__init__('show_kw_urls', ['keyword'],
+                                           db=db)
 	pass
 	
     def __call__(self):
@@ -649,7 +686,7 @@ class show_kw_urls(new_action):
 	form = self.form
 
 	keyword = form['keyword'].decode('utf8')
-	urls = GinGin_db.get_kw_urls(keyword)
+	urls = self.db.get_kw_urls(keyword)
 	content = represent.show_kw_urls(keyword, urls)
 	content['title'] = keyword
 	
@@ -659,8 +696,10 @@ class show_kw_urls(new_action):
 
 ## Unassociate URLs with specified keyword.
 class del_kw_urls(new_action):
-    def __init__(self):
-	super(del_kw_urls, self).__init__('del_kw_urls', ['keyword', 'url_id'])
+    def __init__(self, db):
+	super(del_kw_urls, self).__init__('del_kw_urls',
+                                          ['keyword', 'url_id'],
+                                          db=db)
 	
     def __call__(self):
 	import types
@@ -675,7 +714,7 @@ class del_kw_urls(new_action):
                 url_ids = [url_ids]
                 pass
             url_ids = map(lambda x: int(x), url_ids)
-            GinGin_db.del_kw_urls(keyword, url_ids)
+            self.db.del_kw_urls(keyword, url_ids)
         else:
             url_ids = []
             pass
@@ -683,11 +722,11 @@ class del_kw_urls(new_action):
 
 ## Show document links for spider.
 class show_to_robot(new_action):
-    def __init__(self):
-	super(show_to_robot, self).__init__('show_to_robot', [])
+    def __init__(self, db):
+	super(show_to_robot, self).__init__('show_to_robot', [], db=db)
 	
     def __call__(self):
-	docs = GinGin_db.get_last_doc_ids(GINGIN_TO_ROBOT_N)
+	docs = self.db.get_last_doc_ids(GINGIN_TO_ROBOT_N)
 	represent.show_to_robot(docs, self.env)
 	pass
     pass
@@ -695,14 +734,14 @@ class show_to_robot(new_action):
 
 ## Return RSS feed of the GinGin system.
 class rssfeed(new_action):
-    def __init__(self):
-	super(rssfeed, self).__init__('rssfeed', [])
+    def __init__(self, db):
+	super(rssfeed, self).__init__('rssfeed', [], db=db)
 	pass
     
     def __call__(self):
 	from templates import rssfeed
 	
-	docs = GinGin_db.get_last_abstracts(GINGIN_TO_RSSFEED_N)
+	docs = self.db.get_last_abstracts(GINGIN_TO_RSSFEED_N)
 	docs = map(lambda x: (x[0], x[1], gingin_date_to_rss_date(x[2]), x[3], x[4]), docs)
 	self.octr.set_media_type('application/xml')
 	content = represent.show_rssfeed(docs)
@@ -715,8 +754,11 @@ class rssfeed(new_action):
 
 ## Update/add an URL.
 class update_url(new_action):
-    def __init__(self):
-	super(update_url, self).__init__('update_url', ['keywords', 'url_id', 'title', 'url', 'abstract'])
+    def __init__(self, db):
+	super(update_url, self).__init__('update_url',
+                                         ['keywords', 'url_id', 'title',
+                                          'url', 'abstract'],
+                                         db=db)
 	pass
 	
     def __call__(self):
@@ -742,7 +784,7 @@ class update_url(new_action):
         except KeyError:
 	    return represent.error_page('paramter error', self.octr.ofo)
 	uid = GINGIN_USER.get_id_from_user(self, user)
-	url_id = GinGin_db.update_url(url_id, title, url, abstract, uid, kws)
+	url_id = self.db.update_url(url_id, title, url, abstract, uid, kws)
 	base, spath = tools.get_server_url(self.env)
 	tools.redirect(self.octr.ofo, '%s/show_kw_urls/%s' % (spath, str(kws[0].decode('utf-8'))))
 	pass
@@ -751,8 +793,8 @@ class update_url(new_action):
 
 ## Show a form for adding a new URL.
 class edit_new_url(new_action):
-    def __init__(self):
-	super(edit_new_url, self).__init__('edit_new_url', [])
+    def __init__(self, db):
+	super(edit_new_url, self).__init__('edit_new_url', [], db=db)
 	pass
 	
     def __call__(self):
@@ -768,15 +810,15 @@ class edit_new_url(new_action):
 
 ## Show a form for editing an exisited URL.
 class edit_url(new_action):
-    def __init__(self):
-	super(edit_url, self).__init__('edit_url', ['url_id'])
+    def __init__(self, db):
+	super(edit_url, self).__init__('edit_url', ['url_id'], db=db)
 	pass
     
     def __call__(self):
 	from templates import edit_url
 	
 	url_id = int(self.form['url_id'])
-	url, kws = GinGin_db.get_url(url_id)
+	url, kws = self.db.get_url(url_id)
 	
 	content = {'title': 'Edit a URL',
                    'url_id': url_id,
@@ -792,15 +834,16 @@ class edit_url(new_action):
 
 ## Show a form to create a new comment for the document with specified ID.
 class edit_comment(new_action):
-    def __init__(self):
-	super(edit_comment, self).__init__('edit_comment', ['doc_id'])
+    def __init__(self, db):
+	super(edit_comment, self).__init__('edit_comment', ['doc_id'],
+                                           db=db)
 	pass
     
     def __call__(self):
 	from templates import comment
 	
 	doc_id = int(self.form['doc_id'])
-	attack = GinGin_db.get_id_doc_attack(doc_id)
+	attack = self.db.get_id_doc_attack(doc_id)
 	content = {'title': 'Edit Comment', 'doc_id': doc_id, 'attack_plain': attack[1], 'attack_cipher': attack[2]}
 	
 	comment.str = tools.hack_str
@@ -810,8 +853,11 @@ class edit_comment(new_action):
 
 ## Add a comment to the document with specified ID.
 class add_comment(new_action):
-    def __init__(self):
-	super(add_comment, self).__init__('add_comment', ['doc_id', 'key', 'name', 'email', 'website', 'msg'])
+    def __init__(self, db):
+	super(add_comment, self).__init__('add_comment',
+                                          ['doc_id', 'key', 'name',
+                                           'email', 'website', 'msg'],
+                                          db=db)
 	pass
     
     def __call__(self):
@@ -829,7 +875,7 @@ class add_comment(new_action):
 	    website = None
 	    pass
 	msg = form['msg']
-	GinGin_db.add_id_doc_comment(doc_id, name, email, website, msg, key)
+	self.db.add_id_doc_comment(doc_id, name, email, website, msg, key)
 	
         base, spath = get_server_url(self.octr.env)
         represent.show_update_ok(doc_id, '', spath)
@@ -838,13 +884,13 @@ class add_comment(new_action):
 
 ## Show lastest comments of the GinGin system.
 class last_comments(new_action):
-    def __init__(self):
-	super(last_comments, self).__init__('last_comments', [])
+    def __init__(self, db):
+	super(last_comments, self).__init__('last_comments', [], db=db)
 	pass
     
     def __call__(self):
 	from templates import last_comments
-	comments = GinGin_db.get_last_comments(GINGIN_LAST_COMMENTS_N)
+	comments = self.db.get_last_comments(GINGIN_LAST_COMMENTS_N)
 	cmts = [{'cmt_id': c[0], 'doc_id': c[1], 'name': c[2], 'website': c[4], 'msg': c[5], 'pdate': c[6]} for c in comments]
 	content = {'title': 'Last Comments', 'comments': cmts}
 	
@@ -854,8 +900,9 @@ class last_comments(new_action):
 
 ## Show trackback of a specified document.
 class trackback(new_action):
-    def __init__(self):
-	super(trackback, self).__init__('trackback', ['doc_id', '__mod'])
+    def __init__(self, db):
+	super(trackback, self).__init__('trackback', ['doc_id', '__mod'],
+                                        db=db)
         pass
 
     def __call__(self):
@@ -868,12 +915,12 @@ class trackback(new_action):
 	    __mod = form['__mod']
 	    if __mod in ('rss', 'view'):
 		kws, doc_id, title, doc_body, pdate, uid, published = \
-                    GinGin_db.get_id_doc(doc_id)
+                    self.db.get_id_doc(doc_id)
 		desc = doc_body
 		if len(desc) > GINGIN_TB_BODY_MAX:
 		    desc = desc[:GINGIN_TB_BODY_MAX] + '...'
 		link = get_url_of_doc(doc_id, self.env)
-		pings = GinGin_db.get_trackback_pings(doc_id)
+		pings = self.db.get_trackback_pings(doc_id)
 		if __mod == 'rss':
                     from templates import tb_rss
                     content = {'title': title,
@@ -942,7 +989,7 @@ class trackback(new_action):
             else:
                 rhost = self.env['REMOTE_ADDR'] + ':' + self.env['REMOTE_PORT']
                 if exist_doc_id(doc_id):
-                    GinGin_db.trackback_ping(doc_id, title, excerpt, url, blog_name, rhost)
+                    self.db.trackback_ping(doc_id, title, excerpt, url, blog_name, rhost)
                     emsg = None
                 else:
                     emsg = 'doc %d is not found' % (doc_id)
@@ -967,8 +1014,8 @@ class trackback(new_action):
 ## @}
 
 def full_side_bar_info(out):
-    docs = GinGin_db.get_last_docs(GINGIN_MOST_RECENT_N)
-    hot_kws = GinGin_db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
+    docs = self.db.get_last_docs(GINGIN_MOST_RECENT_N)
+    hot_kws = self.db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
     
     hks = map(lambda x: {'name': x[0], 'num': int(x[1])}, hot_kws)
     lds = map(lambda x: {'id': x[0], 'title': x[1]}, docs)
@@ -1012,7 +1059,12 @@ def _show_error_page(form, env, octr):
     #                                       repr(f.f_globals))
     return represent.error_page(emsg1, octr.ofo)
 
-def GinGin_run_action(form, env, octr):
+## \brief Actually run actions.
+#
+# Request from WEB server are handled by CGI/FastCGI handler and
+# passed to this function.  This function inspects content of the
+# request and invokes respective action.
+def GinGin_run_action(form, env, octr, db):
     import mez_xml.tools
     mez_xml.tools.str = tools.hack_str
     
@@ -1024,7 +1076,7 @@ def GinGin_run_action(form, env, octr):
     act_clazz = _all_actions_.setdefault(func, default_act)
     
     try:
-        act = act_clazz()
+        act = act_clazz(db)
         act.set_form(form, env)
         act.set_octr(octr)
         return act()
@@ -1040,6 +1092,7 @@ def GinGin_run_action(form, env, octr):
 
 def _init_actions():
     global _all_actions_
+
     _all_actions_ = {}
     for name, obj in globals().items():
         if type(obj) == type and \
