@@ -4,12 +4,11 @@ def list_pair_to_pairs_list(l1, l2):
     assert(len(l1) == len(l2))
     return [(l1[i], l2[i]) for i in range(len(l1))]
 
-def get_server_url():
-    import os
-    spath = 'http://' + os.environ['SERVER_NAME']
-    if os.environ['SERVER_PORT'] != '80':
-	spath = spath + ':' + os.environ['SERVER_PORT']
-    spath = spath + os.environ['SCRIPT_NAME']
+def get_server_url(env):
+    spath = 'http://' + env['SERVER_NAME']
+    if env['SERVER_PORT'] != '80':
+	spath = spath + ':' + env['SERVER_PORT']
+    spath = spath + env['SCRIPT_NAME']
     last = spath.rfind('/')
     base = spath[:last+1]
     return base, spath
