@@ -14,8 +14,13 @@ class REDIRECT:
 
 # DONE: Use 'env' from octr instead of 'os.environ'.
 class anony_user:
-    def __init__(self):
+    def __init__(self, db=None):
+        self._db = db
 	pass
+
+    def set_db(self, db):
+        self._db = db
+        pass
     
     def get_login_user(self, octr):
 	return 'guest'
@@ -41,9 +46,17 @@ class anony_user:
 
 
 class HTTP_auth:
-    def __init__(self):
+    def __init__(self, db=None):
+        self._db = db
 	pass
     
+    ## \brief Setup database
+    #
+    # \note This supposed to be invoked by CGI entry.
+    def set_db(self, db):
+        self._db = db
+        pass
+
     def get_login_user(self, octr):
 	import os
 	user = None
@@ -93,7 +106,7 @@ class HTTP_auth:
         if (task in ('show_afiles', 'get_afile', 'show_id_doc',
                      'show_id_doc_src', 'edit_id_doc', 'trackback')) and \
            (method != 'GET' or
-            ((user == None) and (not GinGin_db.is_doc_published(doc_id)))):
+            ((user == None) and (not self._db.is_doc_published(doc_id)))):
             auth_first()
             pass
 	return 1
